@@ -1,10 +1,32 @@
 import json
+from datetime import datetime
 
-from httpx import AsyncClient
+import httpx
 from thingy import NamesMixin, Thingy, classproperty
 
 
+class JSONEncoder(json.JSONEncoder):
+    def default(self, o):
+        if isinstance(o, (datetime.date, datetime.datetime)):
+            return o.isoformat()
+        return json.JSONEncoder.default(self, o)
+
+
+class AsyncClient(httpx.AsyncClient):
+    _json_encoder = JSONEncoder
+
+    async def request(self, *args, **kwargs):
+        if "json" in kwargs:
+            json = kwargs.pop("json")
+            kwargs["content"] = self._json_encoder.dumps(json)
+            kwargs.setdefault("headers", {})
+            kwargs["headers"]["Content-Type"] = "application/json"
+        return await super().request(*args, **kwargs)
+
+
 class Cursor:
+    _json_encoder = JSONEncoder
+
     def __init__(self, cls, params, cache=False):
         self.cls = cls
         self.params = params
@@ -17,12 +39,10 @@ class Cursor:
     async def _get_page(self):
         self.params["cursor"] = self.index
 
-        async with AsyncClient(base_url=self.cls.base_url) as client:
-            params = {key: json.dumps(value) for key, value in self.params.items()}
+        async with self.cls._get_client() as client:
             response = await client.get(
                 f"/api/1.1/obj/{self.cls.typename}",
-                params=params,
-                headers=self.cls._headers,
+                params=self.params,
             )
             response.raise_for_status()
         return response.json()["response"]
@@ -78,6 +98,8 @@ class Cursor:
 
 class BubbleThing(NamesMixin, Thingy):
     _base_url = None
+    _client_cls = AsyncClient
+    _json_encoder = JSONEncoder
     _headers = {}
     _typename = None
 
@@ -115,17 +137,31 @@ class BubbleThing(NamesMixin, Thingy):
     def typename(cls):
         return cls._typename or "".join(cls.names)
 
+    @classmethod
+    def _get_client(cls):
+        return cls._client_cls(
+            base_url=cls._base_url,
+            headers=cls._headers,
+        )
+
+    @classmethod
+    def _dump_params(cls, params):
+        for key, value in params.items():
+            if not isinstance(value, str):
+                params[key] = json.dumps(value, cls=cls._json_encoder)
+
     @classmethod
     def get(cls, **params):
+        cls._dump_params(params)
         return Cursor(cls, params)
 
     @classmethod
     async def _get_by_id(cls, id, **params):
-        async with AsyncClient(base_url=cls.base_url) as client:
+        cls._dump_params(params)
+        async with cls._get_client() as client:
             response = await client.get(
                 f"/api/1.1/obj/{cls.typename}/{id}",
                 params=params,
-                headers=cls._headers,
             )
             response.raise_for_status()
 
@@ -204,22 +240,22 @@ class BubbleThing(NamesMixin, Thingy):
             return await self._join_by_cls(key, cursor_or_other_cls, **params)
 
     async def put(self, **params):
-        async with AsyncClient(base_url=self.__class__._base_url) as client:
+        self._dump_params(params)
+        async with self._get_client() as client:
             response = await client.put(
                 f"/api/1.1/obj/{self.__class__.typename}/{self._id}",
                 params=params,
-                headers=self.__class__._headers,
                 json=self.view("bubble"),
             )
             response.raise_for_status()
         return self
 
     async def post(self, **params):
-        async with AsyncClient(base_url=self.__class__._base_url) as client:
+        self._dump_params(params)
+        async with self._get_client() as client:
             response = await client.post(
                 f"/api/1.1/obj/{self.__class__.typename}",
                 params=params,
-                headers=self.__class__._headers,
                 json=self.view("bubble"),
             )
             response.raise_for_status()
